port module Example.LogIn exposing
    ( Flags
    , Model
    , Msg
    , encodeMsg
    , fromCache
    , init
    , msgDecoder
    , msgFromInt
    , subscriptions
    , update
    , view
    )

import AMR
import Browser
import Html exposing (Html, button, div, input, text)
import Html.Attributes exposing (placeholder, type_, value)
import Html.Events exposing (onClick, onInput)
import Json.Decode as Decode
import Json.Encode as Encode


type alias Flags =
    { amr : Maybe String }


type Model
    = LoggedOut
        { name : String
        , pass : String
        }
    | LoggedIn
        { name : String
        }


type Msg
    = InputName String
    | InputPass String
    | LogIn
    | LogOut


init : Flags -> ( Model, Cmd Msg )
init _ =
    ( LoggedOut
        { name = ""
        , pass = ""
        }
    , Cmd.none
    )


update : Msg -> Model -> ( Model, Cmd Msg )
update msg model =
    case msg of
        InputName name ->
            case model of
                LoggedOut lo ->
                    ( LoggedOut { lo | name = name }, Cmd.none )

                LoggedIn _ ->
                    ( model, Cmd.none )

        InputPass pass ->
            case model of
                LoggedOut lo ->
                    ( LoggedOut { lo | pass = pass }, Cmd.none )

                LoggedIn _ ->
                    ( model, Cmd.none )

        LogIn ->
            case model of
                LoggedOut lo ->
                    ( LoggedIn { name = lo.name }, Cmd.none )

                LoggedIn _ ->
                    ( model, Cmd.none )

        LogOut ->
            case model of
                LoggedIn li ->
                    ( LoggedOut { name = li.name, pass = "" }
                    , Cmd.none
                    )

                LoggedOut _ ->
                    ( model, Cmd.none )


subscriptions : Model -> Sub Msg
subscriptions _ =
    Sub.none


view : Model -> Browser.Document Msg
view model =
    { title = "Login"
    , body =
        case model of
            LoggedOut lo ->
                viewLoggedOut lo

            LoggedIn li ->
                viewLoggedIn li
    }


viewLoggedIn : { name : String } -> List (Html Msg)
viewLoggedIn model =
    [ div [] [ text ("Hello " ++ model.name) ]
    , button [ onClick LogOut ] [ text "Log Out" ]
    ]


viewLoggedOut :
    { name : String, pass : String }
    -> List (Html Msg)
viewLoggedOut model =
    [ input
        [ onInput InputName
        , value model.name
        , placeholder "name"
        ]
        []
    , input
        [ onInput InputPass
        , value model.pass
        , type_ "password"
        , placeholder "password"
        ]
        []
    , button
        [ onClick LogIn
        ]
        [ text "Log In" ]
    ]



-- AMR


fromCache : Flags -> Maybe String
fromCache =
    .amr


msgFromInt : Int -> Msg
msgFromInt n =
    case modBy 4 n of
        1 ->
            InputName (String.fromInt n)

        2 ->
            InputPass (String.fromInt n)

        3 ->
            LogIn

        _ ->
            LogOut


encodeMsg : Msg -> Encode.Value
encodeMsg msg =
    case msg of
        InputName name ->
            Encode.object
                [ ( "InputName", Encode.string name )
                ]

        InputPass pass ->
            Encode.object
                [ ( "InputPass", Encode.string pass )
                ]

        LogIn ->
            Encode.object [ ( "LogIn", Encode.null ) ]

        LogOut ->
            Encode.object [ ( "LogOut", Encode.null ) ]


msgDecoder : Decode.Decoder Msg
msgDecoder =
    Decode.oneOf
        [ Decode.field "InputName"
            (Decode.map InputName Decode.string)
        , Decode.field "InputPass"
            (Decode.map InputPass Decode.string)
        , Decode.field "LogIn" (Decode.null LogIn)
        , Decode.field "LogOut" (Decode.null LogOut)
        ]
